#include "perception.h"
#include "../common/message.h"
#include "../common/canmsgobdmsgid.h"
#include "../canbus/candatatype.h"




namespace  robot {
namespace perception {


ros::Publisher Perception::s_perData_pub;

std::string Perception::Name() const { return "perception"; }

int Perception::Init()
{
    LOG(INFO) << Name() << " Init start...";

    node_handle_.reset(new ros::NodeHandle());

    s_perData_pub = node_handle_->advertise<canbus::canMsg>(robot::common::PerceptionData, 2000);

    LOG(INFO) << Name() << " Init end.";
}

int Perception::Start()
{
    ros::Subscriber sub = node_handle_->subscribe(robot::common::CanDataAnalysis, 1000, CanTopicCallback);
    ros::spin();
}

void Perception::Stop()
{

}


void Perception::CanTopicCallback(const canbus::canMsg::ConstPtr& msg)
{
    LOG(INFO) << " Msg CanData :" << "ID:" << msg->id << ",Data:" << msg->data;

    bool sendFlag = false;

    switch (msg->id) {
    case robot::common::PowerOnReq:
    {
        static robot::canbusSpace::T_CONTROL_CMD_5F1 cmd_5F1_PRO;
        memset(&cmd_5F1_PRO, 0 ,sizeof(cmd_5F1_PRO));

        robot::canbusSpace::T_CONTROL_CMD_5F1 cmd_5F1_REV;
        memcpy(&cmd_5F1_REV, msg->data.c_str(), sizeof(cmd_5F1_REV));

        if(cmd_5F1_REV.uiAccCtrPowReq != cmd_5F1_PRO.uiAccCtrPowReq && cmd_5F1_REV.uiBrkCtrPowReq != cmd_5F1_PRO.uiBrkCtrPowReq &&
                cmd_5F1_REV.uiClchCtrPowReq != cmd_5F1_PRO.uiClchCtrPowReq && cmd_5F1_REV.uiRCUPowReq != cmd_5F1_PRO.uiRCUPowReq &&
                cmd_5F1_REV.uiStrCtrPowReq != cmd_5F1_PRO.uiStrCtrPowReq && cmd_5F1_REV.uiXSftArmCtrPowReq != cmd_5F1_PRO.uiXSftArmCtrPowReq &&
                cmd_5F1_REV.uiYSftArmCtrPowReq != cmd_5F1_PRO.uiYSftArmCtrPowReq)
        {
            sendFlag = true;
        }

        memcpy(&cmd_5F1_PRO, msg->data.c_str(), sizeof(cmd_5F1_PRO));
    }
        break;
    case robot::common::ActuatorEnableStatus:
    {
        static robot::canbusSpace::T_STATE_OUT_6E5 s_state_6E5;
        memset(&s_state_6E5, 0 ,sizeof(s_state_6E5));

        robot::canbusSpace::T_STATE_OUT_6E5 state_6E5;
        memcpy(&state_6E5, msg->data.c_str(), sizeof(state_6E5));

        if(state_6E5.uiAccClchCtrStat != s_state_6E5.uiAccClchCtrStat && state_6E5.uiAccMotEnStat != s_state_6E5.uiAccMotEnStat &&
                state_6E5.uiBrkHVFFStat != s_state_6E5.uiBrkHVFFStat && state_6E5.uiBrkMotEnStat != s_state_6E5.uiBrkMotEnStat &&
                state_6E5.uiClchMotEnStat != s_state_6E5.uiClchMotEnStat && state_6E5.uiPauseEnStat != s_state_6E5.uiPauseEnStat &&
                state_6E5.uiStrMotEnStat != s_state_6E5.uiStrMotEnStat && state_6E5.uiXSftMotEnStat != s_state_6E5.uiXSftMotEnStat &&
                state_6E5.uiYSftMotEnStat != s_state_6E5.uiYSftMotEnStat);
        {
            sendFlag = true;
        }
    }
        break;
    default:
        break;
    }

    if (sendFlag)
    {
        s_perData_pub.publish(msg);
    }
}

}  // namespace perception
}  // namespace robot
